/*
 * Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.  Users and possessors of this source code
 * are hereby granted a nonexclusive, royalty-free license to use this code
 * in individual and commercial software.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.   This source code is a "commercial item" as
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer  software"  and "commercial computer software
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 *
 * Any use of this source code in individual and commercial software must
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

/* Template project which demonstrates the basics on how to setup a project 
* example application.
* Host code.
*/

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include <cutil.h>

////////////////////////////////////////////////////////////////////////////////
// declaration, forward
void runAddImageTest( int argc, char** argv);

////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////
int
main( int argc, char** argv) 
{
    runAddImageTest( argc, argv);

    CUT_EXIT(argc, argv);
}

void runAddImageTest( int argc, char** argv) 
{
	CUT_DEVICE_INIT();

	const int IMG_WIDTH = 352, IMG_HEIGHT = 288;
	const int IMG_STEP = (IMG_WIDTH + 3) &(~0x03);

	unsigned int gpuTimer = 0;

	unsigned char* im1 = (unsigned char*)malloc(IMG_STEP * IMG_HEIGHT);
	unsigned char* im2 = (unsigned char*)malloc(IMG_STEP * IMG_HEIGHT);
	memset(im1, 5, IMG_STEP * IMG_HEIGHT);
	memset(im2, 25, IMG_STEP * IMG_HEIGHT);

    void* d_img1 = 0, *d_img2 = 0, *d_res = 0;
	size_t d_img1Pitch = 0, d_img2Pitch = 0, d_resPitch = 0;
	CUT_SAFE_CALL(cutCreateTimer(&gpuTimer));
	CUT_SAFE_CALL(cutStartTimer(gpuTimer));
	CUDA_SAFE_CALL(cudaMallocPitch((void**)&d_img1, &d_img1Pitch, IMG_WIDTH, IMG_HEIGHT));
	CUDA_SAFE_CALL(cudaMallocPitch((void**)&d_img2, &d_img2Pitch, IMG_WIDTH, IMG_HEIGHT));
	CUDA_SAFE_CALL(cudaMallocPitch((void**)&d_res, &d_resPitch, IMG_WIDTH, IMG_HEIGHT));
	CUDA_SAFE_CALL(cudaMemcpy2D(d_img1, d_img1Pitch, im1, IMG_STEP, IMG_WIDTH, IMG_HEIGHT, cudaMemcpyHostToDevice));
	CUDA_SAFE_CALL(cudaMemcpy2D(d_img2, d_img2Pitch, im2, IMG_STEP, IMG_WIDTH, IMG_HEIGHT, cudaMemcpyHostToDevice));
	CUT_SAFE_CALL(cutStopTimer(gpuTimer));
	printf( "GPU processing time\t: %f (ms)\n", cutGetTimerValue(gpuTimer));
	CUT_SAFE_CALL(cutDeleteTimer(gpuTimer));

	free(im1);
	free(im2);
}
